<?php

/**
 * ECountryCodes.php
 *
 * A simple component to generate a dropDownList with country codes.
 * Download of the data from 'http://opencountrycodes.appspot.com'
 *
 * Based on a code snippet found at
 * @link http://dragffy.com/blog/posts/creating-a-php-array-of-iso-3166-1-country-codes
 *
 * @author Joe Blocher <yii@myticket.at>
 * @copyright 2011 myticket it-solutions gmbh
 * @license New BSD License
 * @category UI
 * @version 1.0
 */
class ECountryCodes extends CComponent
{
	const URL = 'http://opencountrycodes.appspot.com';

	public $filePath; //default is set to applications runtime path
	public $fileName = 'country_codes';
	public $update = false; //set to true to get the latest data

	/**
	 * The formats directly supported for downloads
	 *
	 * Other formats see:
	 * @link http://opencountrycodes.appspot.com
	 *
	 * @var $_formats
	 */
	private $_formats = array('xml','json');

	/**
	 * Create the component
	 *
	 * @param array $config
	 */
	public function __construct($config = array())
	{
		if (!empty($config))
			foreach($config as $key=>$value)
				$this->$key=$value;

		if (empty($this->filePath))
			$this->filePath = Yii::app()->getRuntimePath();
	}


	/**
	 * Absolute path of country codes file
	 *
	 * @return string
	 */
	protected function getCountryCodesFile($extension)
	{
		return $this->filePath . DIRECTORY_SEPARATOR .  $this->fileName . '.' . $extension;
	}


	/**
	 * Download country codes as xml or json
	 *
	 * @return string
	 */
	public function downloadCountryCodes($format='xml',$saveToFile=true)
	{
		if (!in_array($format,$this->_formats))
			throw new CException('Format not supported: ' . $format);

		$data = file_get_contents(self::URL.'/' . $format);

		if (empty($data))
			throw new CException('Error on downloading country code data.');

		if ($saveToFile)
		{
		  $file = $this->getCountryCodesFile($format);
		  file_put_contents($file, $data);
		  if (!is_file($file))
				throw new CException('Error on saving country code data.');
		}

		return $data;
	}

	/**
	 * Get countrycodes as php array(code=>countryName ...)
	 *
	 * @return array
	 */
	public function countryCodesArray()
	{
		$phpArrayFile = $this->getCountryCodesFile('php');

		if (!is_file($phpArrayFile) || $this->update)
		{
			$xmlData = $this->downloadCountryCodes();

			$xml = new SimpleXMLElement($xmlData);
			$out .= '<?php'."\n";
			$out .= 'return array('."\n";

			foreach ($xml->country as $country)
				$out .= " '{$country['code']}' => \"{$country['name']}\",\n";

			$out .= ");";

			file_put_contents($phpArrayFile, $out);

			if (!is_file($phpArrayFile))
				throw new CException('Error on saving php array data.');
		}

		$data = include($phpArrayFile);
		return $data;
	}


	/**
	 * Get countrycodes as XML data
	 *
	 * @return string
	 */
	public function countryCodesXML()
	{
		$file = $this->getCountryCodesFile('xml');

		if (!is_file($file) || $this->update)
		{
			$data = $this->downloadCountryCodes();
			return $data;
		}
		else
			return file_get_contents($file);
	}

	/**
	 * Get countrycodes as JSON data
	 *
	 * @return string
	 */
	public function countryCodesJSON()
	{
		$file = $this->getCountryCodesFile('json');

		if (!is_file($file) || $this->update)
		{
			$data = $this->downloadCountryCodes('json');
			return $data;
		}
		else
			return file_get_contents($xmlFile);
	}


	/**
	 * Move $firstItems to the top of the array
	 *
	 * @param mixed $firstItems
	 * @param mixed $data
	 * @return
	 */
	protected function arrangeFirstItems(&$firstItems,&$data)
	{
		if (!empty($firstItems))
		{
			$sorted = array();
			foreach ($firstItems as $key)
			{
				if (array_key_exists($key,$data))
				{
					$sorted[$key] = $data[$key];
					unset($data[$key]);
				}
			}

			$data = array_merge($sorted,$data);
		}

		return $data;
	}

	/**
	 * Generates a drop down list
	 *
	 * @param string $name the input name
	 * @param string $select the selected value
	 * @param array $firstItems keys of countrycodes listed first
	 * @param array $htmlOptions see CHtml::dropDownList
	 * @return string
	 */
	public function dropDownList($name,$select,$firstItems = array(),$htmlOptions=array())
	{
		$data = $this->arrangeFirstItems($firstItems,$this->countryCodesArray());
		return CHtml::dropDownList($name,$select,$data,$htmlOptions);
	}

	/**
	 * Generates a active drop down list
	 *
	 * @param string $model the data model
	 * @param string $attribute the attribute
	 * @param array $firstItems keys of countrycodes listed first
	 * @param array $htmlOptions see CHtml::activeDropDownList
	 * @return string
	 */
	public function activeDropDownList($model,$attribute,$firstItems = array(),$htmlOptions=array())
	{
	    $data = $this->arrangeFirstItems($firstItems,$this->countryCodesArray());
		return CHtml::activeDropDownList($model,$attribute,$data,$htmlOptions);
	}

}